/*
 * Reading parallel EPROM/EEPROM
 * http://www.kerrywong.com
 * 1/2012
 */
#include <SPI.h>
#include <MCP23S17.h>

//parameters for 16K EPROM
/*const int MAX_ADDR_BITS = 13;
const int MAX_ADDR = 16384;*/

//parameters for 2K EEPROM
const int MAX_ADDR_BITS = 10;
const int MAX_ADDR = 2048;

const int PIN_D0 = 2;
const int PIN_D1 = 3;
const int PIN_D2 = 4;
const int PIN_D3 = 5;
const int PIN_D4 = 6;
const int PIN_D5 = 7;
const int PIN_D6 = 8;
const int PIN_D7 = 9;

const int PIN_WE = 14;
const int PIN_OE = 15;
const int PIN_CE = 16;
MCP mcp(0);

void setReadMode() {
    for (int i = 2; i<= 9; i++) pinMode(i, INPUT);
}

void setWriteMode() {
    for (int i = 2; i<= 9; i++) pinMode(i, OUTPUT);
}

void setup()
{
    Serial.begin(9600);

    for (int i = 14; i<=16; i++) pinMode(i, OUTPUT);
    for (int i = 1; i <= 16; i++) mcp.pinMode(i, LOW);    

    digitalWrite(PIN_WE, HIGH);
    digitalWrite(PIN_OE, HIGH);
    digitalWrite(PIN_CE, HIGH);

    dump();
}

void dump()
{
    delay(10);
    setReadMode();
    delay(10);
    for (int a = 0; a < MAX_ADDR; a++) {

        if (a % 16 == 0) {
            Serial.println();
        } 
        Serial.print(readAddr(a));
        Serial.print("  ");

    }

    Serial.println();
    Serial.println();    
}

byte readAddr(int addr)
{
    for (int a = 0; a <= MAX_ADDR_BITS; a++) {
        mcp.digitalWrite(a + 1, (addr & (1 << a)) >> a);
    }

    delay(1);
    digitalWrite(PIN_CE, LOW);
    delay(1);
    digitalWrite(PIN_OE, LOW);
    delay(1);

    byte o = 0;

    for (int i = 2; i<=9; i++) {
        o = o | (digitalRead(i) << (i - 2));
    }

    digitalWrite(PIN_CE, HIGH);
    delay(1);    
    digitalWrite(PIN_OE, HIGH);
    delay(1);

    return o;
}

void writeAddr(int addr, byte val)
{
    for (int a = 0; a <= MAX_ADDR_BITS; a++) {
        mcp.digitalWrite(a + 1, (addr & (1 << a)) >> a);
    }

    for (int i = 2; i<=9; i++) {
        int a = i - 2;
        digitalWrite(i, (val & (1 << a)) >> a);
    }

    delay(1);
    digitalWrite(PIN_CE, LOW);
    delay(1);
    digitalWrite(PIN_WE, LOW);
    delay(1);
    digitalWrite(PIN_CE, HIGH);
    delay(1);
    digitalWrite(PIN_WE, HIGH);
    delay(1);
}

void loop()
{

}

